/*
 *  FSFreeze.h
 *  FlareStorm: Standard Freeze Data Format Definitions
 *
 *  Created by 2510 on Wed Feb 11 2004.
 *  Copyright (c) 2004 by 2510. All rights reserved.
 *
 */

#ifndef included_FSFreeze
#define included_FSFreeze

/* GPU */
typedef struct FSFreezeGPU
{
	unsigned long			length; /* total length of freeze data */
	
	unsigned char			framebuffer[1024 * 512 * 2];
	FSAsyncTaskID			vsync_async;
	struct
	{
		unsigned char		buffered;
		unsigned long		buffer[16];
	}						command;
	struct
	{
		unsigned char		dma_mode,display_enable;
		unsigned short		screen_mode,draw_mode,mask_mode;
		unsigned long		texture_window;
		unsigned long		range_h,range_v;
	}						status;
	struct
	{
		unsigned short		left,right,
							top,bottom;
	}						clip;
	struct
	{
		mips_word			address; /* logical address */
		mips_word			size; /* block size and block count */
		mips_word			control; /* channel control */
		FSAsyncTaskID		async;
	}						dma,otc;
	
	unsigned char			opaque[0]; /* plug-in specific data */
}	FSFreezeGPU;

/* CD-ROM */
typedef struct FSFreezeCDROM
{
	unsigned long			length; /* total length of freeze data */
	
	struct
	{
		unsigned char		mode,status,
							file,channel;
		unsigned long		location;
	}						cdrom;
	struct
	{
		unsigned char		mode,
							parameters,results,result_ptr,
							parameter[8],result[8];
	}						reg;
	unsigned char			interrupt;
	struct
	{
		unsigned char		buffer[2352];
		unsigned long		buffer_pos;
		FSAsyncTaskID		async;
	}						reading;
	struct
	{
		unsigned char		control,phase,parameters,
							parameter[8];
		unsigned long		wait;
		struct
		{
			unsigned char	control,parameters,
							parameter[8];
		}					next;
		FSAsyncTaskID		async;
	}						control;
	struct
	{
		unsigned char		status; /* 0:closed, 1:opened, 2:opening, 3:closing */
		unsigned char		phase;
		FSAsyncTaskID		async;
	}						shell;
	struct
	{
		mips_word			address; /* logical address */
		mips_word			size; /* block size and block count */
		mips_word			control; /* channel control */
		FSAsyncTaskID		async;
	}						dma;
	struct
	{
		long				last[2][2];
		unsigned char		sleep;
	}						adpcm;
	struct
	{
		unsigned char		playing;
	}						cdda;
	
	unsigned char			opaque[0]; /* plug-in specific data */
}	FSFreezeCDROM;

/* SPU */
typedef struct FSFreezeSPU
{
	unsigned long			length; /* total length of freeze data */
	
	FSAsyncTaskID			async;
	unsigned char			spu_memory[512 * 1024];
	unsigned char			register_memory[0x200];
	struct
	{
		unsigned short		control,status;
		unsigned long		transfer_address;
		
	}						reg;
	struct
	{
		mips_word			address; /* logical address */
		mips_word			size; /* block size and block count */
		mips_word			control; /* channel control */
	}						dma;
	struct
	{
		unsigned short		address;
		unsigned char		pending;
	}						interrupt;
	
	unsigned char			opaque[0]; /* plug-in specific data */
}	FSFreezeSPU;

/* Root Counter */
typedef struct FSFreezeRTC
{
	unsigned long			length; /* total length of freeze data */
	
	struct
	{
		unsigned short		mode,target,count;
		FSAsyncTaskID		async;
	}						rtc[3];
	
	unsigned char			opaque[0]; /* plug-in specific data */
}	FSFreezeRTC;

/* Serial I/O port 0 */
typedef struct FSFreezeSIO0
{
	unsigned long			length; /* total length of freeze data */
	
	struct
	{
		FSAsyncTaskID		async;
		unsigned char		buffered;
		unsigned char		buffer[192];
		
		unsigned short		status,control,mode,baud;
	}						reg;
	struct
	{
		/* InitPAD, StartPAD, StopPAD */
		struct
		{
			mips_word		address; /* logical */
			mips_word		size;
		}					auto_pad[2];
		/* PAD_init, PAD_dr */
		struct
		{
			mips_word		address; /* logical */
		}					man_pad;
	}						service;
	
	unsigned char			opaque[0]; /* plug-in specific data */
}	FSFreezeSIO0;

/* MDEC */
typedef struct FSFreezeMDEC
{
	unsigned long			length; /* total length of freeze data */
	
	struct
	{
		unsigned long		mdec0,mdec1;
	}						reg;
	struct
	{
		int					quantize_y[64],
							quantize_uv[64];
		mips_word			source_address; /* logical */
		mips_word			source_length;
	}						decompression;
	struct
	{
		mips_word			address; /* logical address */
		mips_word			size; /* block size and block count */
		mips_word			control; /* channel control */
		FSAsyncTaskID		async;
	}						dma_in,dma_out;
	
	unsigned char			opaque[0]; /* plug-in specific data */
}	FSFreezeMDEC;

/* System Service */
typedef struct FSFreezeSystem
{
	unsigned long			length; /* total length of freeze data */
	
	mips_address			sysheap_base,sysheap_size;
	mips_address			malloc_base,malloc_size;
	mips_address			strtok_address;
	mips_word				rand_seed;
	
	unsigned char			opaque[0]; /* plug-in specific data */
}	FSFreezeSystem;

#endif